import os
import asyncio
#todo this is not importing right
from ..hcb_soap_client import HcbSoapClient

async def RunTest():
    school = await HcbSoapClient.get_school_info(os.environ["HCB_SCHOOLCODE"])
    schoolId = school.customer.id
    userInfo = await HcbSoapClient.get_parent_info(
        schoolId, os.environ["HCB_USERNAME"], os.environ["HCB_Password"]
    )
    parentId = userInfo.account.id
    studentId = userInfo.linked_students.student[0].entity_id
    stops = await HcbSoapClient.get_bus_info(schoolId, parentId, studentId, HcbSoapClient.AM_ID)
    print(stops.vehicle_location.address)


asyncio.run(RunTest())



from datetime import datetime
from uuid import UUID
from typing import Any, List, TypeVar, Callable, Type, cast
import dateutil.parser


T = TypeVar("T")


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_datetime(x: Any) -> datetime:
    return dateutil.parser.parse(x)


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


class StudentStop:
    name: str
    latitude: str
    longitude: str
    start_time: datetime
    stop_type: str
    substitute_vehicle_name: str
    vehicle_name: str
    stop_id: str
    arrival_time: datetime
    time_of_day_id: str
    vehicle_id: UUID
    esn: str
    tier_start_time: datetime
    bus_visibility_start_offset: int

    def __init__(
        self,
        name: str,
        latitude: str,
        longitude: str,
        start_time: datetime,
        stop_type: str,
        substitute_vehicle_name: str,
        vehicle_name: str,
        stop_id: str,
        arrival_time: datetime,
        time_of_day_id: str,
        vehicle_id: UUID,
        esn: str,
        tier_start_time: datetime,
        bus_visibility_start_offset: int,
    ) -> None:
        self.name = name
        self.latitude = latitude
        self.longitude = longitude
        self.start_time = start_time
        self.stop_type = stop_type
        self.substitute_vehicle_name = substitute_vehicle_name
        self.vehicle_name = vehicle_name
        self.stop_id = stop_id
        self.arrival_time = arrival_time
        self.time_of_day_id = time_of_day_id
        self.vehicle_id = vehicle_id
        self.esn = esn
        self.tier_start_time = tier_start_time
        self.bus_visibility_start_offset = bus_visibility_start_offset

    @staticmethod
    def from_dict(obj: Any) -> "StudentStop":
        assert isinstance(obj, dict)
        name = from_str(obj.get("@Name"))
        latitude = from_str(obj.get("@Latitude"))
        longitude = from_str(obj.get("@Longitude"))
        start_time = from_datetime(obj.get("@StartTime"))
        stop_type = from_str(obj.get("@StopType"))
        substitute_vehicle_name = from_str(obj.get("@SubstituteVehicleName"))
        vehicle_name = from_str(obj.get("@VehicleName"))
        stop_id = from_str(obj.get("@StopId"))
        arrival_time = from_datetime(obj.get("@ArrivalTime"))
        time_of_day_id = from_str(obj.get("@TimeOfDayId"))
        vehicle_id = UUID(obj.get("@VehicleId"))
        esn = from_str(obj.get("@Esn"))
        tier_start_time = from_datetime(obj.get("@TierStartTime"))
        bus_visibility_start_offset = int(
            from_str(obj.get("@BusVisibilityStartOffset"))
        )
        return StudentStop(
            name,
            latitude,
            longitude,
            start_time,
            stop_type,
            substitute_vehicle_name,
            vehicle_name,
            stop_id,
            arrival_time,
            time_of_day_id,
            vehicle_id,
            esn,
            tier_start_time,
            bus_visibility_start_offset,
        )

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Name"] = from_str(self.name)
        result["@Latitude"] = from_str(self.latitude)
        result["@Longitude"] = from_str(self.longitude)
        result["@StartTime"] = self.start_time.isoformat()
        result["@StopType"] = from_str(self.stop_type)
        result["@SubstituteVehicleName"] = from_str(self.substitute_vehicle_name)
        result["@VehicleName"] = from_str(self.vehicle_name)
        result["@StopId"] = from_str(self.stop_id)
        result["@ArrivalTime"] = self.arrival_time.isoformat()
        result["@TimeOfDayId"] = from_str(self.time_of_day_id)
        result["@VehicleId"] = str(self.vehicle_id)
        result["@Esn"] = from_str(self.esn)
        result["@TierStartTime"] = self.tier_start_time.isoformat()
        result["@BusVisibilityStartOffset"] = from_str(
            str(self.bus_visibility_start_offset)
        )
        return result


class StudentStops:
    student_stop: List[StudentStop]

    def __init__(self, student_stop: List[StudentStop]) -> None:
        self.student_stop = student_stop

    @staticmethod
    def from_dict(obj: Any) -> "StudentStops":
        if(obj is None):
            return None
        assert isinstance(obj, dict)
        student_stop = from_list(StudentStop.from_dict, obj.get("StudentStop"))
        return StudentStops(student_stop)

    def to_dict(self) -> dict:
        result: dict = {}
        result["StudentStop"] = from_list(
            lambda x: to_class(StudentStop, x), self.student_stop
        )
        return result


class VehicleLocation:
    name: str
    latitude: str
    longitude: str
    log_time: str
    ignition: str
    latent: str
    time_zone_offset: int
    heading: str
    speed: int
    address: str
    message_code: int
    display_on_map: str

    def __init__(
        self,
        name: str,
        latitude: str,
        longitude: str,
        log_time: str,
        ignition: str,
        latent: str,
        time_zone_offset: int,
        heading: str,
        speed: int,
        address: str,
        message_code: int,
        display_on_map: str,
    ) -> None:
        self.name = name
        self.latitude = latitude
        self.longitude = longitude
        self.log_time = log_time
        self.ignition = ignition
        self.latent = latent
        self.time_zone_offset = time_zone_offset
        self.heading = heading
        self.speed = speed
        self.address = address
        self.message_code = message_code
        self.display_on_map = display_on_map

    @staticmethod
    def from_dict(obj: Any) -> "VehicleLocation":
        if(obj is None):
            return None
        assert isinstance(obj, dict)
        name = from_str(obj.get("@Name"))
        latitude = from_str(obj.get("@Latitude"))
        longitude = from_str(obj.get("@Longitude"))
        log_time = from_str(obj.get("@LogTime"))
        ignition = from_str(obj.get("@Ignition"))
        latent = from_str(obj.get("@Latent"))
        time_zone_offset = int(from_str(obj.get("@TimeZoneOffset")))
        heading = from_str(obj.get("@Heading"))
        speed = int(from_str(obj.get("@Speed")))
        address = from_str(obj.get("@Address"))
        message_code = int(from_str(obj.get("@MessageCode")))
        display_on_map = from_str(obj.get("@DisplayOnMap"))
        return VehicleLocation(
            name,
            latitude,
            longitude,
            log_time,
            ignition,
            latent,
            time_zone_offset,
            heading,
            speed,
            address,
            message_code,
            display_on_map,
        )

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Name"] = from_str(self.name)
        result["@Latitude"] = from_str(self.latitude)
        result["@Longitude"] = from_str(self.longitude)
        result["@LogTime"] = from_str(self.log_time)
        result["@Ignition"] = from_str(self.ignition)
        result["@Latent"] = from_str(self.latent)
        result["@TimeZoneOffset"] = from_str(str(self.time_zone_offset))
        result["@Heading"] = from_str(self.heading)
        result["@Speed"] = from_str(str(self.speed))
        result["@Address"] = from_str(self.address)
        result["@MessageCode"] = from_str(str(self.message_code))
        result["@DisplayOnMap"] = from_str(self.display_on_map)
        return result


class GetStudentStops:
    vehicle_location: VehicleLocation
    student_stops: StudentStops

    def __init__(
        self, vehicle_location: VehicleLocation, student_stops: StudentStops
    ) -> None:
        self.vehicle_location = vehicle_location
        self.student_stops = student_stops

    @staticmethod
    def from_dict(obj: Any) -> "GetStudentStops":
        assert isinstance(obj, dict)
        vehicle_location = VehicleLocation.from_dict(obj.get("VehicleLocation"))
        student_stops = StudentStops.from_dict(obj.get("StudentStops"))
        return GetStudentStops(vehicle_location, student_stops)

    def to_dict(self) -> dict:
        result: dict = {}
        result["VehicleLocation"] = to_class(VehicleLocation, self.vehicle_location)
        result["StudentStops"] = to_class(StudentStops, self.student_stops)
        return result


class GetStudentStopsAndScans:
    get_student_stops: GetStudentStops

    def __init__(self, get_student_stops: GetStudentStops) -> None:
        self.get_student_stops = get_student_stops

    @staticmethod
    def from_dict(obj: Any) -> "GetStudentStopsAndScans":
        assert isinstance(obj, dict)
        get_student_stops = GetStudentStops.from_dict(obj.get("GetStudentStops"))
        return GetStudentStopsAndScans(get_student_stops)

    def to_dict(self) -> dict:
        result: dict = {}
        result["GetStudentStops"] = to_class(GetStudentStops, self.get_student_stops)
        return result


class Status:
    code: int
    message: str
    next_id: int
    next_call_time: str

    def __init__(
        self, code: int, message: str, next_id: int, next_call_time: str
    ) -> None:
        self.code = code
        self.message = message
        self.next_id = next_id
        self.next_call_time = next_call_time

    @staticmethod
    def from_dict(obj: Any) -> "Status":
        assert isinstance(obj, dict)
        code = int(from_str(obj.get("@Code")))
        message = from_str(obj.get("@Message"))
        next_id = int(from_str(obj.get("@NextID")))
        next_call_time = from_str(obj.get("@NextCallTime"))
        return Status(code, message, next_id, next_call_time)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Code"] = from_str(str(self.code))
        result["@Message"] = from_str(self.message)
        result["@NextID"] = from_str(str(self.next_id))
        result["@NextCallTime"] = from_str(self.next_call_time)
        return result


class SynoviaAPI:
    xsi_schema_location: str
    version: str
    xmlns: str
    xmlns_xsi: str
    status: Status
    get_student_stops_and_scans: GetStudentStopsAndScans

    def __init__(
        self,
        xsi_schema_location: str,
        version: str,
        xmlns: str,
        xmlns_xsi: str,
        status: Status,
        get_student_stops_and_scans: GetStudentStopsAndScans,
    ) -> None:
        self.xsi_schema_location = xsi_schema_location
        self.version = version
        self.xmlns = xmlns
        self.xmlns_xsi = xmlns_xsi
        self.status = status
        self.get_student_stops_and_scans = get_student_stops_and_scans

    @staticmethod
    def from_dict(obj: Any) -> "SynoviaAPI":
        assert isinstance(obj, dict)
        xsi_schema_location = from_str(obj.get("@xsi:schemaLocation"))
        version = from_str(obj.get("@Version"))
        xmlns = from_str(obj.get("@xmlns"))
        xmlns_xsi = from_str(obj.get("@xmlns:xsi"))
        status = Status.from_dict(obj.get("Status"))
        get_student_stops_and_scans = GetStudentStopsAndScans.from_dict(
            obj.get("GetStudentStopsAndScans")
        )
        return SynoviaAPI(
            xsi_schema_location,
            version,
            xmlns,
            xmlns_xsi,
            status,
            get_student_stops_and_scans,
        )

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xsi:schemaLocation"] = from_str(self.xsi_schema_location)
        result["@Version"] = from_str(self.version)
        result["@xmlns"] = from_str(self.xmlns)
        result["@xmlns:xsi"] = from_str(self.xmlns_xsi)
        result["Status"] = to_class(Status, self.status)
        result["GetStudentStopsAndScans"] = to_class(
            GetStudentStopsAndScans, self.get_student_stops_and_scans
        )
        return result


class S1158Result:
    synovia_api: SynoviaAPI

    def __init__(self, synovia_api: SynoviaAPI) -> None:
        self.synovia_api = synovia_api

    @staticmethod
    def from_dict(obj: Any) -> "S1158Result":
        assert isinstance(obj, dict)
        synovia_api = SynoviaAPI.from_dict(obj.get("SynoviaApi"))
        return S1158Result(synovia_api)

    def to_dict(self) -> dict:
        result: dict = {}
        result["SynoviaApi"] = to_class(SynoviaAPI, self.synovia_api)
        return result


class S1158Response:
    xmlns: str
    s1158_result: S1158Result

    def __init__(self, xmlns: str, s1158_result: S1158Result) -> None:
        self.xmlns = xmlns
        self.s1158_result = s1158_result

    @staticmethod
    def from_dict(obj: Any) -> "S1158Response":
        assert isinstance(obj, dict)
        xmlns = from_str(obj.get("@xmlns"))
        s1158_result = S1158Result.from_dict(obj.get("s1158Result"))
        return S1158Response(xmlns, s1158_result)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xmlns"] = from_str(self.xmlns)
        result["s1158Result"] = to_class(S1158Result, self.s1158_result)
        return result


class SBody:
    s1158_response: S1158Response

    def __init__(self, s1158_response: S1158Response) -> None:
        self.s1158_response = s1158_response

    @staticmethod
    def from_dict(obj: Any) -> "SBody":
        assert isinstance(obj, dict)
        s1158_response = S1158Response.from_dict(obj.get("s1158Response"))
        return SBody(s1158_response)

    def to_dict(self) -> dict:
        result: dict = {}
        result["s1158Response"] = to_class(S1158Response, self.s1158_response)
        return result


class SEnvelope:
    xmlns_s: str
    s_body: SBody

    def __init__(self, xmlns_s: str, s_body: SBody) -> None:
        self.xmlns_s = xmlns_s
        self.s_body = s_body

    @staticmethod
    def from_dict(obj: Any) -> "SEnvelope":
        assert isinstance(obj, dict)
        xmlns_s = from_str(obj.get("@xmlns:s"))
        s_body = SBody.from_dict(obj.get("s:Body"))
        return SEnvelope(xmlns_s, s_body)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xmlns:s"] = from_str(self.xmlns_s)
        result["s:Body"] = to_class(SBody, self.s_body)
        return result


class S1158:
    s_envelope: SEnvelope

    def __init__(self, s_envelope: SEnvelope) -> None:
        self.s_envelope = s_envelope

    @staticmethod
    def from_dict(obj: Any) -> "S1158":
        assert isinstance(obj, dict)
        s_envelope = SEnvelope.from_dict(obj.get("s:Envelope"))
        return S1158(s_envelope)

    def to_dict(self) -> dict:
        result: dict = {}
        result["s:Envelope"] = to_class(SEnvelope, self.s_envelope)
        return result


def S1158_from_dict(s: Any) -> S1158:
    return S1158.from_dict(s)


def S1158_to_dict(x: S1158) -> Any:
    return to_class(S1158, x)



from typing import Any, List, TypeVar, Callable, Type, cast


T = TypeVar("T")


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


class Status:
    code: int
    message: str
    next_id: int
    next_call_time: str

    def __init__(self, code: int, message: str, next_id: int, next_call_time: str) -> None:
        self.code = code
        self.message = message
        self.next_id = next_id
        self.next_call_time = next_call_time

    @staticmethod
    def from_dict(obj: Any) -> 'Status':
        assert isinstance(obj, dict)
        code = int(from_str(obj.get("@Code")))
        message = from_str(obj.get("@Message"))
        next_id = int(from_str(obj.get("@NextID")))
        next_call_time = from_str(obj.get("@NextCallTime"))
        return Status(code, message, next_id, next_call_time)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Code"] = from_str(str(self.code))
        result["@Message"] = from_str(self.message)
        result["@NextID"] = from_str(str(self.next_id))
        result["@NextCallTime"] = from_str(self.next_call_time)
        return result


class Carrier:
    id: int
    name: str

    def __init__(self, id: int, name: str) -> None:
        self.id = id
        self.name = name

    @staticmethod
    def from_dict(obj: Any) -> 'Carrier':
        assert isinstance(obj, dict)
        id = int(from_str(obj.get("@ID")))
        name = from_str(obj.get("@Name"))
        return Carrier(id, name)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@ID"] = from_str(str(self.id))
        result["@Name"] = from_str(self.name)
        return result


class CarrierList:
    carrier: List[Carrier]

    def __init__(self, carrier: List[Carrier]) -> None:
        self.carrier = carrier

    @staticmethod
    def from_dict(obj: Any) -> 'CarrierList':
        assert isinstance(obj, dict)
        carrier = from_list(Carrier.from_dict, obj.get("Carrier"))
        return CarrierList(carrier)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Carrier"] = from_list(lambda x: to_class(Carrier, x), self.carrier)
        return result


class ContactType:
    name: str

    def __init__(self, name: str) -> None:
        self.name = name

    @staticmethod
    def from_dict(obj: Any) -> 'ContactType':
        assert isinstance(obj, dict)
        name = from_str(obj.get("@Name"))
        return ContactType(name)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Name"] = from_str(self.name)
        return result


class ContactTypeList:
    contact_type: List[ContactType]

    def __init__(self, contact_type: List[ContactType]) -> None:
        self.contact_type = contact_type

    @staticmethod
    def from_dict(obj: Any) -> 'ContactTypeList':
        assert isinstance(obj, dict)
        contact_type = from_list(ContactType.from_dict, obj.get("ContactType"))
        return ContactTypeList(contact_type)

    def to_dict(self) -> dict:
        result: dict = {}
        result["ContactType"] = from_list(lambda x: to_class(ContactType, x), self.contact_type)
        return result


class Customer:
    name: str
    id: str
    planned_routing_system: str
    snapshot_start_date: str
    snapshot_id: int
    last_import_run_date: str
    last_import_status: str
    hctb_minimum_radius: int
    hctb_maximum_radius: int
    hctb_sso_type: str
    hctb_setting_registration_enabled: str
    hctb_setting_password_reset_enabled: str
    hctb_trim_leading_zeroes_in_the_app: str
    hctb_display_boarding_card_number_below_barcode: str
    local_id: int
    host: str
    ecommerce_url: str
    store_id: str
    api_token: str
    profile_id: str
    admin_email: str
    customer_email: str

    def __init__(self, name: str, id: str, planned_routing_system: str, snapshot_start_date: str, snapshot_id: int, last_import_run_date: str, last_import_status: str, hctb_minimum_radius: int, hctb_maximum_radius: int, hctb_sso_type: str, hctb_setting_registration_enabled: str, hctb_setting_password_reset_enabled: str, hctb_trim_leading_zeroes_in_the_app: str, hctb_display_boarding_card_number_below_barcode: str, local_id: int, host: str, ecommerce_url: str, store_id: str, api_token: str, profile_id: str, admin_email: str, customer_email: str) -> None:
        self.name = name
        self.id = id
        self.planned_routing_system = planned_routing_system
        self.snapshot_start_date = snapshot_start_date
        self.snapshot_id = snapshot_id
        self.last_import_run_date = last_import_run_date
        self.last_import_status = last_import_status
        self.hctb_minimum_radius = hctb_minimum_radius
        self.hctb_maximum_radius = hctb_maximum_radius
        self.hctb_sso_type = hctb_sso_type
        self.hctb_setting_registration_enabled = hctb_setting_registration_enabled
        self.hctb_setting_password_reset_enabled = hctb_setting_password_reset_enabled
        self.hctb_trim_leading_zeroes_in_the_app = hctb_trim_leading_zeroes_in_the_app
        self.hctb_display_boarding_card_number_below_barcode = hctb_display_boarding_card_number_below_barcode
        self.local_id = local_id
        self.host = host
        self.ecommerce_url = ecommerce_url
        self.store_id = store_id
        self.api_token = api_token
        self.profile_id = profile_id
        self.admin_email = admin_email
        self.customer_email = customer_email

    @staticmethod
    def from_dict(obj: Any) -> 'Customer':
        assert isinstance(obj, dict)
        name = from_str(obj.get("@Name"))
        id = from_str(obj.get("@ID"))
        planned_routing_system = from_str(obj.get("@PlannedRoutingSystem"))
        snapshot_start_date = from_str(obj.get("@SnapshotStartDate"))
        snapshot_id = int(from_str(obj.get("@SnapshotId")))
        last_import_run_date = from_str(obj.get("@LastImportRunDate"))
        last_import_status = from_str(obj.get("@LastImportStatus"))
        hctb_minimum_radius = int(from_str(obj.get("@HctbMinimumRadius")))
        hctb_maximum_radius = int(from_str(obj.get("@HctbMaximumRadius")))
        hctb_sso_type = from_str(obj.get("@HctbSsoType"))
        hctb_setting_registration_enabled = from_str(obj.get("@HctbSettingRegistrationEnabled"))
        hctb_setting_password_reset_enabled = from_str(obj.get("@HctbSettingPasswordResetEnabled"))
        hctb_trim_leading_zeroes_in_the_app = from_str(obj.get("@HctbTrimLeadingZeroesInTheApp"))
        hctb_display_boarding_card_number_below_barcode = from_str(obj.get("@HctbDisplayBoardingCardNumberBelowBarcode"))
        local_id = int(from_str(obj.get("@LocalId")))
        host = from_str(obj.get("@Host"))
        ecommerce_url = from_str(obj.get("@EcommerceUrl"))
        store_id = from_str(obj.get("@StoreId"))
        api_token = from_str(obj.get("@ApiToken"))
        profile_id = from_str(obj.get("@ProfileId"))
        admin_email = from_str(obj.get("@AdminEmail"))
        customer_email = from_str(obj.get("@CustomerEmail"))
        return Customer(name, id, planned_routing_system, snapshot_start_date, snapshot_id, last_import_run_date, last_import_status, hctb_minimum_radius, hctb_maximum_radius, hctb_sso_type, hctb_setting_registration_enabled, hctb_setting_password_reset_enabled, hctb_trim_leading_zeroes_in_the_app, hctb_display_boarding_card_number_below_barcode, local_id, host, ecommerce_url, store_id, api_token, profile_id, admin_email, customer_email)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Name"] = from_str(self.name)
        result["@ID"] = from_str(self.id)
        result["@PlannedRoutingSystem"] = from_str(self.planned_routing_system)
        result["@SnapshotStartDate"] = from_str(self.snapshot_start_date)
        result["@SnapshotId"] = from_str(str(self.snapshot_id))
        result["@LastImportRunDate"] = from_str(self.last_import_run_date)
        result["@LastImportStatus"] = from_str(self.last_import_status)
        result["@HctbMinimumRadius"] = from_str(str(self.hctb_minimum_radius))
        result["@HctbMaximumRadius"] = from_str(str(self.hctb_maximum_radius))
        result["@HctbSsoType"] = from_str(self.hctb_sso_type)
        result["@HctbSettingRegistrationEnabled"] = from_str(self.hctb_setting_registration_enabled)
        result["@HctbSettingPasswordResetEnabled"] = from_str(self.hctb_setting_password_reset_enabled)
        result["@HctbTrimLeadingZeroesInTheApp"] = from_str(self.hctb_trim_leading_zeroes_in_the_app)
        result["@HctbDisplayBoardingCardNumberBelowBarcode"] = from_str(self.hctb_display_boarding_card_number_below_barcode)
        result["@LocalId"] = from_str(str(self.local_id))
        result["@Host"] = from_str(self.host)
        result["@EcommerceUrl"] = from_str(self.ecommerce_url)
        result["@StoreId"] = from_str(self.store_id)
        result["@ApiToken"] = from_str(self.api_token)
        result["@ProfileId"] = from_str(self.profile_id)
        result["@AdminEmail"] = from_str(self.admin_email)
        result["@CustomerEmail"] = from_str(self.customer_email)
        return result


class LicenseList:
    license: List[ContactType]

    def __init__(self, license: List[ContactType]) -> None:
        self.license = license

    @staticmethod
    def from_dict(obj: Any) -> 'LicenseList':
        assert isinstance(obj, dict)
        license = from_list(ContactType.from_dict, obj.get("License"))
        return LicenseList(license)

    def to_dict(self) -> dict:
        result: dict = {}
        result["License"] = from_list(lambda x: to_class(ContactType, x), self.license)
        return result


class NotificationLanguageList:
    notification_language: List[ContactType]

    def __init__(self, notification_language: List[ContactType]) -> None:
        self.notification_language = notification_language

    @staticmethod
    def from_dict(obj: Any) -> 'NotificationLanguageList':
        assert isinstance(obj, dict)
        notification_language = from_list(ContactType.from_dict, obj.get("NotificationLanguage"))
        return NotificationLanguageList(notification_language)

    def to_dict(self) -> dict:
        result: dict = {}
        result["NotificationLanguage"] = from_list(lambda x: to_class(ContactType, x), self.notification_language)
        return result


class NotificationTypeList:
    notification_type: List[ContactType]

    def __init__(self, notification_type: List[ContactType]) -> None:
        self.notification_type = notification_type

    @staticmethod
    def from_dict(obj: Any) -> 'NotificationTypeList':
        assert isinstance(obj, dict)
        notification_type = from_list(ContactType.from_dict, obj.get("NotificationType"))
        return NotificationTypeList(notification_type)

    def to_dict(self) -> dict:
        result: dict = {}
        result["NotificationType"] = from_list(lambda x: to_class(ContactType, x), self.notification_type)
        return result


class ValidateCustomerAccountNumber:
    customer: Customer
    license_list: LicenseList
    carrier_list: CarrierList
    contact_type_list: ContactTypeList
    notification_language_list: NotificationLanguageList
    notification_type_list: NotificationTypeList

    def __init__(self, customer: Customer, license_list: LicenseList, carrier_list: CarrierList, contact_type_list: ContactTypeList, notification_language_list: NotificationLanguageList, notification_type_list: NotificationTypeList) -> None:
        self.customer = customer
        self.license_list = license_list
        self.carrier_list = carrier_list
        self.contact_type_list = contact_type_list
        self.notification_language_list = notification_language_list
        self.notification_type_list = notification_type_list

    @staticmethod
    def from_dict(obj: Any) -> 'ValidateCustomerAccountNumber':
        assert isinstance(obj, dict)
        customer = Customer.from_dict(obj.get("Customer"))
        license_list = LicenseList.from_dict(obj.get("LicenseList"))
        carrier_list = CarrierList.from_dict(obj.get("CarrierList"))
        contact_type_list = ContactTypeList.from_dict(obj.get("ContactTypeList"))
        notification_language_list = NotificationLanguageList.from_dict(obj.get("NotificationLanguageList"))
        notification_type_list = NotificationTypeList.from_dict(obj.get("NotificationTypeList"))
        return ValidateCustomerAccountNumber(customer, license_list, carrier_list, contact_type_list, notification_language_list, notification_type_list)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Customer"] = to_class(Customer, self.customer)
        result["LicenseList"] = to_class(LicenseList, self.license_list)
        result["CarrierList"] = to_class(CarrierList, self.carrier_list)
        result["ContactTypeList"] = to_class(ContactTypeList, self.contact_type_list)
        result["NotificationLanguageList"] = to_class(NotificationLanguageList, self.notification_language_list)
        result["NotificationTypeList"] = to_class(NotificationTypeList, self.notification_type_list)
        return result


class SynoviaAPI:
    xsi_schema_location: str
    version: str
    xmlns: str
    xmlns_xsi: str
    status: Status
    validate_customer_account_number: ValidateCustomerAccountNumber

    def __init__(self, xsi_schema_location: str, version: str, xmlns: str, xmlns_xsi: str, status: Status, validate_customer_account_number: ValidateCustomerAccountNumber) -> None:
        self.xsi_schema_location = xsi_schema_location
        self.version = version
        self.xmlns = xmlns
        self.xmlns_xsi = xmlns_xsi
        self.status = status
        self.validate_customer_account_number = validate_customer_account_number

    @staticmethod
    def from_dict(obj: Any) -> 'SynoviaAPI':
        assert isinstance(obj, dict)
        xsi_schema_location = from_str(obj.get("@xsi:schemaLocation"))
        version = from_str(obj.get("@Version"))
        xmlns = from_str(obj.get("@xmlns"))
        xmlns_xsi = from_str(obj.get("@xmlns:xsi"))
        status = Status.from_dict(obj.get("Status"))
        validate_customer_account_number = ValidateCustomerAccountNumber.from_dict(obj.get("ValidateCustomerAccountNumber"))
        return SynoviaAPI(xsi_schema_location, version, xmlns, xmlns_xsi, status, validate_customer_account_number)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xsi:schemaLocation"] = from_str(self.xsi_schema_location)
        result["@Version"] = from_str(self.version)
        result["@xmlns"] = from_str(self.xmlns)
        result["@xmlns:xsi"] = from_str(self.xmlns_xsi)
        result["Status"] = to_class(Status, self.status)
        result["ValidateCustomerAccountNumber"] = to_class(ValidateCustomerAccountNumber, self.validate_customer_account_number)
        return result


class S1100Result:
    synovia_api: SynoviaAPI

    def __init__(self, synovia_api: SynoviaAPI) -> None:
        self.synovia_api = synovia_api

    @staticmethod
    def from_dict(obj: Any) -> 'S1100Result':
        assert isinstance(obj, dict)
        synovia_api = SynoviaAPI.from_dict(obj.get("SynoviaApi"))
        return S1100Result(synovia_api)

    def to_dict(self) -> dict:
        result: dict = {}
        result["SynoviaApi"] = to_class(SynoviaAPI, self.synovia_api)
        return result


class S1100Response:
    xmlns: str
    s1100_result: S1100Result

    def __init__(self, xmlns: str, s1100_result: S1100Result) -> None:
        self.xmlns = xmlns
        self.s1100_result = s1100_result

    @staticmethod
    def from_dict(obj: Any) -> 'S1100Response':
        assert isinstance(obj, dict)
        xmlns = from_str(obj.get("@xmlns"))
        s1100_result = S1100Result.from_dict(obj.get("s1100Result"))
        return S1100Response(xmlns, s1100_result)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xmlns"] = from_str(self.xmlns)
        result["s1100Result"] = to_class(S1100Result, self.s1100_result)
        return result


class SBody:
    s1100_response: S1100Response

    def __init__(self, s1100_response: S1100Response) -> None:
        self.s1100_response = s1100_response

    @staticmethod
    def from_dict(obj: Any) -> 'SBody':
        assert isinstance(obj, dict)
        s1100_response = S1100Response.from_dict(obj.get("s1100Response"))
        return SBody(s1100_response)

    def to_dict(self) -> dict:
        result: dict = {}
        result["s1100Response"] = to_class(S1100Response, self.s1100_response)
        return result


class SEnvelope:
    xmlns_s: str
    s_body: SBody

    def __init__(self, xmlns_s: str, s_body: SBody) -> None:
        self.xmlns_s = xmlns_s
        self.s_body = s_body

    @staticmethod
    def from_dict(obj: Any) -> 'SEnvelope':
        assert isinstance(obj, dict)
        xmlns_s = from_str(obj.get("@xmlns:s"))
        s_body = SBody.from_dict(obj.get("s:Body"))
        return SEnvelope(xmlns_s, s_body)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xmlns:s"] = from_str(self.xmlns_s)
        result["s:Body"] = to_class(SBody, self.s_body)
        return result


class S1100:
    s_envelope: SEnvelope

    def __init__(self, s_envelope: SEnvelope) -> None:
        self.s_envelope = s_envelope

    @staticmethod
    def from_dict(obj: Any) -> 'S1100':
        assert isinstance(obj, dict)
        s_envelope = SEnvelope.from_dict(obj.get("s:Envelope"))
        return S1100(s_envelope)

    def to_dict(self) -> dict:
        result: dict = {}
        result["s:Envelope"] = to_class(SEnvelope, self.s_envelope)
        return result


def s1100_from_dict(s: Any) -> S1100:
    return S1100.from_dict(s)


def s1100_to_dict(x: S1100) -> Any:
    return to_class(S1100, x)



from typing import Any, List, TypeVar, Callable, Type, cast
from datetime import datetime
import dateutil.parser


T = TypeVar("T")


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


def from_datetime(x: Any) -> datetime:
    return dateutil.parser.parse(x)


class Account:
    id: str
    contact_email: str
    carrier: int
    carrier_name: str
    contact_mobile_phone: str
    first_name: str
    last_name: str
    initial_map_lat_lon: str
    notification_language: str
    hctb_minimum_radius: int
    hctb_maximum_radius: int
    hctb_app_timeout: int
    hctb_time_limit_after_gps_activity: int
    hctb_hide_bus_on_indicator: int
    hctb_trim_leading_zeroes_in_the_app: str
    hctb_display_boarding_card_number_below_barcode: str

    def __init__(self, id: str, contact_email: str, carrier: int, carrier_name: str, contact_mobile_phone: str, first_name: str, last_name: str, initial_map_lat_lon: str, notification_language: str, hctb_minimum_radius: int, hctb_maximum_radius: int, hctb_app_timeout: int, hctb_time_limit_after_gps_activity: int, hctb_hide_bus_on_indicator: int, hctb_trim_leading_zeroes_in_the_app: str, hctb_display_boarding_card_number_below_barcode: str) -> None:
        self.id = id
        self.contact_email = contact_email
        self.carrier = carrier
        self.carrier_name = carrier_name
        self.contact_mobile_phone = contact_mobile_phone
        self.first_name = first_name
        self.last_name = last_name
        self.initial_map_lat_lon = initial_map_lat_lon
        self.notification_language = notification_language
        self.hctb_minimum_radius = hctb_minimum_radius
        self.hctb_maximum_radius = hctb_maximum_radius
        self.hctb_app_timeout = hctb_app_timeout
        self.hctb_time_limit_after_gps_activity = hctb_time_limit_after_gps_activity
        self.hctb_hide_bus_on_indicator = hctb_hide_bus_on_indicator
        self.hctb_trim_leading_zeroes_in_the_app = hctb_trim_leading_zeroes_in_the_app
        self.hctb_display_boarding_card_number_below_barcode = hctb_display_boarding_card_number_below_barcode

    @staticmethod
    def from_dict(obj: Any) -> 'Account':
        assert isinstance(obj, dict)
        id = from_str(obj.get("@ID"))
        contact_email = from_str(obj.get("@ContactEmail"))
        carrier = int(from_str(obj.get("@Carrier")))
        carrier_name = from_str(obj.get("@CarrierName"))
        contact_mobile_phone = from_str(obj.get("@ContactMobilePhone"))
        first_name = from_str(obj.get("@FirstName"))
        last_name = from_str(obj.get("@LastName"))
        initial_map_lat_lon = from_str(obj.get("@InitialMapLatLon"))
        notification_language = from_str(obj.get("@NotificationLanguage"))
        hctb_minimum_radius = int(from_str(obj.get("@HctbMinimumRadius")))
        hctb_maximum_radius = int(from_str(obj.get("@HctbMaximumRadius")))
        hctb_app_timeout = int(from_str(obj.get("@HctbAppTimeout")))
        hctb_time_limit_after_gps_activity = int(from_str(obj.get("@HctbTimeLimitAfterGpsActivity")))
        hctb_hide_bus_on_indicator = int(from_str(obj.get("@HctbHideBusOnIndicator")))
        hctb_trim_leading_zeroes_in_the_app = from_str(obj.get("@HctbTrimLeadingZeroesInTheApp"))
        hctb_display_boarding_card_number_below_barcode = from_str(obj.get("@HctbDisplayBoardingCardNumberBelowBarcode"))
        return Account(id, contact_email, carrier, carrier_name, contact_mobile_phone, first_name, last_name, initial_map_lat_lon, notification_language, hctb_minimum_radius, hctb_maximum_radius, hctb_app_timeout, hctb_time_limit_after_gps_activity, hctb_hide_bus_on_indicator, hctb_trim_leading_zeroes_in_the_app, hctb_display_boarding_card_number_below_barcode)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@ID"] = from_str(self.id)
        result["@ContactEmail"] = from_str(self.contact_email)
        result["@Carrier"] = from_str(str(self.carrier))
        result["@CarrierName"] = from_str(self.carrier_name)
        result["@ContactMobilePhone"] = from_str(self.contact_mobile_phone)
        result["@FirstName"] = from_str(self.first_name)
        result["@LastName"] = from_str(self.last_name)
        result["@InitialMapLatLon"] = from_str(self.initial_map_lat_lon)
        result["@NotificationLanguage"] = from_str(self.notification_language)
        result["@HctbMinimumRadius"] = from_str(str(self.hctb_minimum_radius))
        result["@HctbMaximumRadius"] = from_str(str(self.hctb_maximum_radius))
        result["@HctbAppTimeout"] = from_str(str(self.hctb_app_timeout))
        result["@HctbTimeLimitAfterGpsActivity"] = from_str(str(self.hctb_time_limit_after_gps_activity))
        result["@HctbHideBusOnIndicator"] = from_str(str(self.hctb_hide_bus_on_indicator))
        result["@HctbTrimLeadingZeroesInTheApp"] = from_str(self.hctb_trim_leading_zeroes_in_the_app)
        result["@HctbDisplayBoardingCardNumberBelowBarcode"] = from_str(self.hctb_display_boarding_card_number_below_barcode)
        return result


class Carrier:
    id: int
    name: str

    def __init__(self, id: int, name: str) -> None:
        self.id = id
        self.name = name

    @staticmethod
    def from_dict(obj: Any) -> 'Carrier':
        assert isinstance(obj, dict)
        id = int(from_str(obj.get("@ID")))
        name = from_str(obj.get("@Name"))
        return Carrier(id, name)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@ID"] = from_str(str(self.id))
        result["@Name"] = from_str(self.name)
        return result


class CarrierList:
    carrier: List[Carrier]

    def __init__(self, carrier: List[Carrier]) -> None:
        self.carrier = carrier

    @staticmethod
    def from_dict(obj: Any) -> 'CarrierList':
        assert isinstance(obj, dict)
        carrier = from_list(Carrier.from_dict, obj.get("Carrier"))
        return CarrierList(carrier)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Carrier"] = from_list(lambda x: to_class(Carrier, x), self.carrier)
        return result


class Contact:
    contact_id: int
    name: str
    contact_address: str
    contact_type: str
    is_active: str

    def __init__(self, contact_id: int, name: str, contact_address: str, contact_type: str, is_active: str) -> None:
        self.contact_id = contact_id
        self.name = name
        self.contact_address = contact_address
        self.contact_type = contact_type
        self.is_active = is_active

    @staticmethod
    def from_dict(obj: Any) -> 'Contact':
        assert isinstance(obj, dict)
        contact_id = int(from_str(obj.get("@ContactId")))
        name = from_str(obj.get("@Name"))
        contact_address = from_str(obj.get("@ContactAddress"))
        contact_type = from_str(obj.get("@ContactType"))
        is_active = from_str(obj.get("@IsActive"))
        return Contact(contact_id, name, contact_address, contact_type, is_active)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@ContactId"] = from_str(str(self.contact_id))
        result["@Name"] = from_str(self.name)
        result["@ContactAddress"] = from_str(self.contact_address)
        result["@ContactType"] = from_str(self.contact_type)
        result["@IsActive"] = from_str(self.is_active)
        return result


class ContactList:
    contact: List[Contact]

    def __init__(self, contact: List[Contact]) -> None:
        self.contact = contact

    @staticmethod
    def from_dict(obj: Any) -> 'ContactList':
        assert isinstance(obj, dict)
        contact = from_list(Contact.from_dict, obj.get("Contact"))
        return ContactList(contact)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Contact"] = from_list(lambda x: to_class(Contact, x), self.contact)
        return result


class ContactType:
    name: str

    def __init__(self, name: str) -> None:
        self.name = name

    @staticmethod
    def from_dict(obj: Any) -> 'ContactType':
        assert isinstance(obj, dict)
        name = from_str(obj.get("@Name"))
        return ContactType(name)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Name"] = from_str(self.name)
        return result


class ContactTypeList:
    contact_type: List[ContactType]

    def __init__(self, contact_type: List[ContactType]) -> None:
        self.contact_type = contact_type

    @staticmethod
    def from_dict(obj: Any) -> 'ContactTypeList':
        assert isinstance(obj, dict)
        contact_type = from_list(ContactType.from_dict, obj.get("ContactType"))
        return ContactTypeList(contact_type)

    def to_dict(self) -> dict:
        result: dict = {}
        result["ContactType"] = from_list(lambda x: to_class(ContactType, x), self.contact_type)
        return result


class LicenseList:
    license: List[ContactType]

    def __init__(self, license: List[ContactType]) -> None:
        self.license = license

    @staticmethod
    def from_dict(obj: Any) -> 'LicenseList':
        assert isinstance(obj, dict)
        license = from_list(ContactType.from_dict, obj.get("License"))
        return LicenseList(license)

    def to_dict(self) -> dict:
        result: dict = {}
        result["License"] = from_list(lambda x: to_class(ContactType, x), self.license)
        return result


class Feature:
    name: str
    cost: str

    def __init__(self, name: str, cost: str) -> None:
        self.name = name
        self.cost = cost

    @staticmethod
    def from_dict(obj: Any) -> 'Feature':
        assert isinstance(obj, dict)
        name = from_str(obj.get("Name"))
        cost = from_str(obj.get("Cost"))
        return Feature(name, cost)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Name"] = from_str(self.name)
        result["Cost"] = from_str(self.cost)
        return result


class Features:
    feature: List[Feature]

    def __init__(self, feature: List[Feature]) -> None:
        self.feature = feature

    @staticmethod
    def from_dict(obj: Any) -> 'Features':
        assert isinstance(obj, dict)
        feature = from_list(Feature.from_dict, obj.get("Feature"))
        return Features(feature)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Feature"] = from_list(lambda x: to_class(Feature, x), self.feature)
        return result


class License:
    license_id: int
    parent_license_id: int
    name: str
    cost: str
    start_date: datetime
    end_date: datetime
    next_bill_date: datetime
    free_trial: str
    status: str
    billing_freq_months: int
    commitment_months: int
    auto_renew: str
    features: Features

    def __init__(self, license_id: int, parent_license_id: int, name: str, cost: str, start_date: datetime, end_date: datetime, next_bill_date: datetime, free_trial: str, status: str, billing_freq_months: int, commitment_months: int, auto_renew: str, features: Features) -> None:
        self.license_id = license_id
        self.parent_license_id = parent_license_id
        self.name = name
        self.cost = cost
        self.start_date = start_date
        self.end_date = end_date
        self.next_bill_date = next_bill_date
        self.free_trial = free_trial
        self.status = status
        self.billing_freq_months = billing_freq_months
        self.commitment_months = commitment_months
        self.auto_renew = auto_renew
        self.features = features

    @staticmethod
    def from_dict(obj: Any) -> 'License':
        assert isinstance(obj, dict)
        license_id = int(from_str(obj.get("@LicenseId")))
        parent_license_id = int(from_str(obj.get("@ParentLicenseId")))
        name = from_str(obj.get("@Name"))
        cost = from_str(obj.get("@Cost"))
        start_date = from_datetime(obj.get("@StartDate"))
        end_date = from_datetime(obj.get("@EndDate"))
        next_bill_date = from_datetime(obj.get("@NextBillDate"))
        free_trial = from_str(obj.get("@FreeTrial"))
        status = from_str(obj.get("@Status"))
        billing_freq_months = int(from_str(obj.get("@BillingFreqMonths")))
        commitment_months = int(from_str(obj.get("@CommitmentMonths")))
        auto_renew = from_str(obj.get("@AutoRenew"))
        features = Features.from_dict(obj.get("Features"))
        return License(license_id, parent_license_id, name, cost, start_date, end_date, next_bill_date, free_trial, status, billing_freq_months, commitment_months, auto_renew, features)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@LicenseId"] = from_str(str(self.license_id))
        result["@ParentLicenseId"] = from_str(str(self.parent_license_id))
        result["@Name"] = from_str(self.name)
        result["@Cost"] = from_str(self.cost)
        result["@StartDate"] = self.start_date.isoformat()
        result["@EndDate"] = self.end_date.isoformat()
        result["@NextBillDate"] = self.next_bill_date.isoformat()
        result["@FreeTrial"] = from_str(self.free_trial)
        result["@Status"] = from_str(self.status)
        result["@BillingFreqMonths"] = from_str(str(self.billing_freq_months))
        result["@CommitmentMonths"] = from_str(str(self.commitment_months))
        result["@AutoRenew"] = from_str(self.auto_renew)
        result["Features"] = to_class(Features, self.features)
        return result


class Licenses:
    license: License

    def __init__(self, license: License) -> None:
        self.license = license

    @staticmethod
    def from_dict(obj: Any) -> 'Licenses':
        assert isinstance(obj, dict)
        license = License.from_dict(obj.get("License"))
        return Licenses(license)

    def to_dict(self) -> dict:
        result: dict = {}
        result["License"] = to_class(License, self.license)
        return result


class Student:
    entity_id: str
    first_name: str
    last_name: str
    school_id: int
    parent_student_id: str

    def __init__(self, entity_id: str, first_name: str, last_name: str, school_id: int, parent_student_id: str) -> None:
        self.entity_id = entity_id
        self.first_name = first_name
        self.last_name = last_name
        self.school_id = school_id
        self.parent_student_id = parent_student_id

    @staticmethod
    def from_dict(obj: Any) -> 'Student':
        assert isinstance(obj, dict)
        entity_id = from_str(obj.get("@EntityID"))
        first_name = from_str(obj.get("@FirstName"))
        last_name = from_str(obj.get("@LastName"))
        school_id = int(from_str(obj.get("@SchoolId")))
        parent_student_id = from_str(obj.get("@ParentStudentId"))
        return Student(entity_id, first_name, last_name, school_id, parent_student_id)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@EntityID"] = from_str(self.entity_id)
        result["@FirstName"] = from_str(self.first_name)
        result["@LastName"] = from_str(self.last_name)
        result["@SchoolId"] = from_str(str(self.school_id))
        result["@ParentStudentId"] = from_str(self.parent_student_id)
        return result


class LinkedStudents:
    student: List[Student]

    def __init__(self, student: List[Student]) -> None:
        self.student = student

    @staticmethod
    def from_dict(obj: Any) -> 'LinkedStudents':
        assert isinstance(obj, dict)
        student = from_list(Student.from_dict, obj.get("Student"))
        return LinkedStudents(student)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Student"] = from_list(lambda x: to_class(Student, x), self.student)
        return result


class NotificationTypeList:
    notification_type: List[ContactType]

    def __init__(self, notification_type: List[ContactType]) -> None:
        self.notification_type = notification_type

    @staticmethod
    def from_dict(obj: Any) -> 'NotificationTypeList':
        assert isinstance(obj, dict)
        notification_type = from_list(ContactType.from_dict, obj.get("NotificationType"))
        return NotificationTypeList(notification_type)

    def to_dict(self) -> dict:
        result: dict = {}
        result["NotificationType"] = from_list(lambda x: to_class(ContactType, x), self.notification_type)
        return result


class Row:
    image_url: str
    redirect_url: str

    def __init__(self, image_url: str, redirect_url: str) -> None:
        self.image_url = image_url
        self.redirect_url = redirect_url

    @staticmethod
    def from_dict(obj: Any) -> 'Row':
        assert isinstance(obj, dict)
        image_url = from_str(obj.get("@ImageUrl"))
        redirect_url = from_str(obj.get("@RedirectUrl"))
        return Row(image_url, redirect_url)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@ImageUrl"] = from_str(self.image_url)
        result["@RedirectUrl"] = from_str(self.redirect_url)
        return result


class Setting:
    name: str
    row: Row

    def __init__(self, name: str, row: Row) -> None:
        self.name = name
        self.row = row

    @staticmethod
    def from_dict(obj: Any) -> 'Setting':
        assert isinstance(obj, dict)
        name = from_str(obj.get("@Name"))
        row = Row.from_dict(obj.get("row"))
        return Setting(name, row)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Name"] = from_str(self.name)
        result["row"] = to_class(Row, self.row)
        return result


class Settings:
    setting: List[Setting]

    def __init__(self, setting: List[Setting]) -> None:
        self.setting = setting

    @staticmethod
    def from_dict(obj: Any) -> 'Settings':
        assert isinstance(obj, dict)
        setting = from_list(Setting.from_dict, obj.get("Setting"))
        return Settings(setting)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Setting"] = from_list(lambda x: to_class(Setting, x), self.setting)
        return result


class TimeOfDay:
    id: str
    name: str
    begin_time: datetime
    end_time: datetime

    def __init__(self, id: str, name: str, begin_time: datetime, end_time: datetime) -> None:
        self.id = id
        self.name = name
        self.begin_time = begin_time
        self.end_time = end_time

    @staticmethod
    def from_dict(obj: Any) -> 'TimeOfDay':
        assert isinstance(obj, dict)
        id = from_str(obj.get("@ID"))
        name = from_str(obj.get("@Name"))
        begin_time = from_datetime(obj.get("@BeginTime"))
        end_time = from_datetime(obj.get("@EndTime"))
        return TimeOfDay(id, name, begin_time, end_time)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@ID"] = from_str(self.id)
        result["@Name"] = from_str(self.name)
        result["@BeginTime"] = self.begin_time.isoformat()
        result["@EndTime"] = self.end_time.isoformat()
        return result


class TimeOfDays:
    time_of_day: List[TimeOfDay]

    def __init__(self, time_of_day: List[TimeOfDay]) -> None:
        self.time_of_day = time_of_day

    @staticmethod
    def from_dict(obj: Any) -> 'TimeOfDays':
        assert isinstance(obj, dict)
        time_of_day = from_list(TimeOfDay.from_dict, obj.get("TimeOfDay"))
        return TimeOfDays(time_of_day)

    def to_dict(self) -> dict:
        result: dict = {}
        result["TimeOfDay"] = from_list(lambda x: to_class(TimeOfDay, x), self.time_of_day)
        return result


class ParentLogin:
    account: Account
    licenses: Licenses
    carrier_list: CarrierList
    time_of_days: TimeOfDays
    linked_students: LinkedStudents
    contact_list: ContactList
    contact_type_list: ContactTypeList
    license_list: LicenseList
    notification_type_list: NotificationTypeList
    settings: Settings

    def __init__(self, account: Account, licenses: Licenses, carrier_list: CarrierList, time_of_days: TimeOfDays, linked_students: LinkedStudents, contact_list: ContactList, contact_type_list: ContactTypeList, license_list: LicenseList, notification_type_list: NotificationTypeList, settings: Settings) -> None:
        self.account = account
        self.licenses = licenses
        self.carrier_list = carrier_list
        self.time_of_days = time_of_days
        self.linked_students = linked_students
        self.contact_list = contact_list
        self.contact_type_list = contact_type_list
        self.license_list = license_list
        self.notification_type_list = notification_type_list
        self.settings = settings

    @staticmethod
    def from_dict(obj: Any) -> 'ParentLogin':
        assert isinstance(obj, dict)
        account = Account.from_dict(obj.get("Account"))
        licenses = Licenses.from_dict(obj.get("Licenses"))
        carrier_list = CarrierList.from_dict(obj.get("CarrierList"))
        time_of_days = TimeOfDays.from_dict(obj.get("TimeOfDays"))
        linked_students = LinkedStudents.from_dict(obj.get("LinkedStudents"))
        contact_list = ContactList.from_dict(obj.get("ContactList"))
        contact_type_list = ContactTypeList.from_dict(obj.get("ContactTypeList"))
        license_list = LicenseList.from_dict(obj.get("LicenseList"))
        notification_type_list = NotificationTypeList.from_dict(obj.get("NotificationTypeList"))
        settings = Settings.from_dict(obj.get("Settings"))
        return ParentLogin(account, licenses, carrier_list, time_of_days, linked_students, contact_list, contact_type_list, license_list, notification_type_list, settings)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Account"] = to_class(Account, self.account)
        result["Licenses"] = to_class(Licenses, self.licenses)
        result["CarrierList"] = to_class(CarrierList, self.carrier_list)
        result["TimeOfDays"] = to_class(TimeOfDays, self.time_of_days)
        result["LinkedStudents"] = to_class(LinkedStudents, self.linked_students)
        result["ContactList"] = to_class(ContactList, self.contact_list)
        result["ContactTypeList"] = to_class(ContactTypeList, self.contact_type_list)
        result["LicenseList"] = to_class(LicenseList, self.license_list)
        result["NotificationTypeList"] = to_class(NotificationTypeList, self.notification_type_list)
        result["Settings"] = to_class(Settings, self.settings)
        return result


class Status:
    code: int
    message: str
    next_id: int
    next_call_time: str

    def __init__(self, code: int, message: str, next_id: int, next_call_time: str) -> None:
        self.code = code
        self.message = message
        self.next_id = next_id
        self.next_call_time = next_call_time

    @staticmethod
    def from_dict(obj: Any) -> 'Status':
        assert isinstance(obj, dict)
        code = int(from_str(obj.get("@Code")))
        message = from_str(obj.get("@Message"))
        next_id = int(from_str(obj.get("@NextID")))
        next_call_time = from_str(obj.get("@NextCallTime"))
        return Status(code, message, next_id, next_call_time)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@Code"] = from_str(str(self.code))
        result["@Message"] = from_str(self.message)
        result["@NextID"] = from_str(str(self.next_id))
        result["@NextCallTime"] = from_str(self.next_call_time)
        return result


class SynoviaAPI:
    xsi_schema_location: str
    version: str
    xmlns: str
    xmlns_xsi: str
    status: Status
    parent_login: ParentLogin

    def __init__(self, xsi_schema_location: str, version: str, xmlns: str, xmlns_xsi: str, status: Status, parent_login: ParentLogin) -> None:
        self.xsi_schema_location = xsi_schema_location
        self.version = version
        self.xmlns = xmlns
        self.xmlns_xsi = xmlns_xsi
        self.status = status
        self.parent_login = parent_login

    @staticmethod
    def from_dict(obj: Any) -> 'SynoviaAPI':
        assert isinstance(obj, dict)
        xsi_schema_location = from_str(obj.get("@xsi:schemaLocation"))
        version = from_str(obj.get("@Version"))
        xmlns = from_str(obj.get("@xmlns"))
        xmlns_xsi = from_str(obj.get("@xmlns:xsi"))
        status = Status.from_dict(obj.get("Status"))
        parent_login = ParentLogin.from_dict(obj.get("ParentLogin"))
        return SynoviaAPI(xsi_schema_location, version, xmlns, xmlns_xsi, status, parent_login)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xsi:schemaLocation"] = from_str(self.xsi_schema_location)
        result["@Version"] = from_str(self.version)
        result["@xmlns"] = from_str(self.xmlns)
        result["@xmlns:xsi"] = from_str(self.xmlns_xsi)
        result["Status"] = to_class(Status, self.status)
        result["ParentLogin"] = to_class(ParentLogin, self.parent_login)
        return result


class S1157Result:
    synovia_api: SynoviaAPI

    def __init__(self, synovia_api: SynoviaAPI) -> None:
        self.synovia_api = synovia_api

    @staticmethod
    def from_dict(obj: Any) -> 'S1157Result':
        assert isinstance(obj, dict)
        synovia_api = SynoviaAPI.from_dict(obj.get("SynoviaApi"))
        return S1157Result(synovia_api)

    def to_dict(self) -> dict:
        result: dict = {}
        result["SynoviaApi"] = to_class(SynoviaAPI, self.synovia_api)
        return result


class S1157Response:
    xmlns: str
    s1157_result: S1157Result

    def __init__(self, xmlns: str, s1157_result: S1157Result) -> None:
        self.xmlns = xmlns
        self.s1157_result = s1157_result

    @staticmethod
    def from_dict(obj: Any) -> 'S1157Response':
        assert isinstance(obj, dict)
        xmlns = from_str(obj.get("@xmlns"))
        s1157_result = S1157Result.from_dict(obj.get("s1157Result"))
        return S1157Response(xmlns, s1157_result)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xmlns"] = from_str(self.xmlns)
        result["s1157Result"] = to_class(S1157Result, self.s1157_result)
        return result


class SBody:
    s1157_response: S1157Response

    def __init__(self, s1157_response: S1157Response) -> None:
        self.s1157_response = s1157_response

    @staticmethod
    def from_dict(obj: Any) -> 'SBody':
        assert isinstance(obj, dict)
        s1157_response = S1157Response.from_dict(obj.get("s1157Response"))
        return SBody(s1157_response)

    def to_dict(self) -> dict:
        result: dict = {}
        result["s1157Response"] = to_class(S1157Response, self.s1157_response)
        return result


class SEnvelope:
    xmlns_s: str
    s_body: SBody

    def __init__(self, xmlns_s: str, s_body: SBody) -> None:
        self.xmlns_s = xmlns_s
        self.s_body = s_body

    @staticmethod
    def from_dict(obj: Any) -> 'SEnvelope':
        assert isinstance(obj, dict)
        xmlns_s = from_str(obj.get("@xmlns:s"))
        s_body = SBody.from_dict(obj.get("s:Body"))
        return SEnvelope(xmlns_s, s_body)

    def to_dict(self) -> dict:
        result: dict = {}
        result["@xmlns:s"] = from_str(self.xmlns_s)
        result["s:Body"] = to_class(SBody, self.s_body)
        return result


class S1157:
    s_envelope: SEnvelope

    def __init__(self, s_envelope: SEnvelope) -> None:
        self.s_envelope = s_envelope

    @staticmethod
    def from_dict(obj: Any) -> 'S1157':
        assert isinstance(obj, dict)
        s_envelope = SEnvelope.from_dict(obj.get("s:Envelope"))
        return S1157(s_envelope)

    def to_dict(self) -> dict:
        result: dict = {}
        result["s:Envelope"] = to_class(SEnvelope, self.s_envelope)
        return result


def S1157_from_dict(s: Any) -> S1157:
    return S1157.from_dict(s)


def S1157_to_dict(x: S1157) -> Any:
    return to_class(S1157, x)



from xml.sax.saxutils import escape

import aiohttp
import xmltodict

from .s1100 import S1100, ValidateCustomerAccountNumber
from .s1157 import S1157, ParentLogin
from .s1158 import S1158, GetStudentStops


class HcbSoapClient:
    _url = "https://api.synovia.com/SynoviaApi.svc"

    AM_ID = "55632A13-35C5-4169-B872-F5ABDC25DF6A"
    PM_ID = "6E7A050E-0295-4200-8EDC-3611BB5DE1C1"

    def _get_soap_header(self) -> str:
        """Return the soap header."""
        payload = '<?xml version="1.0" encoding="utf-8"?>'
        payload += '<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">'
        payload += "<soap:Body>"
        return payload

    def _get_soap_footer(self) -> str:
        """Return the soap footer."""
        payload = "</soap:Body>"
        payload += "</soap:Envelope>"
        return payload

    def _get_standard_headers(self) -> str:
        """Return standard headers."""
        return {
            "app-version": "3.6.0",
            "app-name": "hctb",
            "client-version": "3.6.0",
            "user-agent": "hctb/3.6.0 App-Press/3.6.0",
            "cache-control": "no-cache",
            "content-type": "text/xml",
            "host": "api.synovia.com",
            "connection": "Keep-Alive",
            "accept-encoding": "gzip",
            "cookie": "SRV=prdweb1",
        }

    async def get_school_info(self, schoolCode: str) -> ValidateCustomerAccountNumber:
        """Return the school info from the api."""
        payload = self._get_soap_header()
        payload += '<s1100 xmlns="http://tempuri.org/">'
        payload += "<P1>" + schoolCode + "</P1>"
        payload += "</s1100>"
        payload += self._get_soap_footer()
        headers = self._get_standard_headers()
        headers["soapaction"] = "http://tempuri.org/ISynoviaApi/s1100"
        async with (
            aiohttp.ClientSession() as session,
            session.post(self._url, data=payload, headers=headers) as response,
        ):
            if response.status != 200:
                return None
            text = await response.text()
            root = S1100.from_dict(xmltodict.parse(text))
            return root.s_envelope.s_body.s1100_response.s1100_result.synovia_api.validate_customer_account_number

    async def get_parent_info(
        self, schoolId: str, username: str, password: str
    ) -> ParentLogin:
        """Return the user info from the api."""
        payload = self._get_soap_header()
        payload += '<s1157 xmlns="http://tempuri.org/">'
        payload += "<P1>" + schoolId + "</P1>"
        payload += "<P2>" + username + "</P2>"
        payload += "<P3>" + escape(password) + "</P3>"
        payload += "<P4>LookupItem_Source_Android</P4>"
        payload += "<P5>Android</P5>"
        payload += "<P6>3.6.0</P6>"
        payload += "<P7/>"
        payload += "</s1157>"
        payload += self._get_soap_footer()
        headers = self._get_standard_headers()
        headers["soapaction"] = "http://tempuri.org/ISynoviaApi/s1157"

        async with (
            aiohttp.ClientSession() as session,
            session.post(self._url, data=payload, headers=headers) as response,
        ):
            if response.status != 200:
                return None
            text = await response.text()
            root = S1157.from_dict(xmltodict.parse(text, force_list={"Student"}))
            return root.s_envelope.s_body.s1157_response.s1157_result.synovia_api.parent_login

    async def get_bus_info(
        self, schoolId: str, parentId: str, studentId: str, timeOfDayId: str
    ) -> GetStudentStops:
        """Return the bus info from the api."""
        payload = self._get_soap_header()
        payload += '<s1158 xmlns="http://tempuri.org/">'
        payload += "<P1>" + schoolId + "</P1>"
        payload += "<P2>" + parentId + "</P2>"
        payload += "<P3>" + studentId + "</P3>"
        payload += "<P4>" + timeOfDayId + "</P4>"
        payload += "<P5>true</P5>"
        payload += "<P6>false</P6>"
        payload += "<P7>10</P7>"
        payload += "<P8>14</P8>"
        payload += "<P9>english</P9>"
        payload += "</s1158>"
        payload += self._get_soap_footer()
        headers = self._get_standard_headers()
        headers["soapaction"] = "http://tempuri.org/ISynoviaApi/s1158"
        async with (
            aiohttp.ClientSession() as session,
            session.post(self._url, data=payload, headers=headers) as response,
        ):
            if response.status != 200:
                return None
            text = await response.text()
            root = S1158.from_dict(xmltodict.parse(text))
            return root.s_envelope.s_body.s1158_response.s1158_result.synovia_api.get_student_stops_and_scans.get_student_stops

    async def test_connection(
        self, school_code: str, user_name: str, password: str
    ) -> bool:
        """Test the connection to the api."""
        school = await self.get_school_info(school_code)
        school_id = school.customer.id
        userInfo = await self.get_parent_info(school_id, user_name, password)
        return userInfo is not None
